module io.dir {
   check(Pathname) :-
      io.file:type(Pathname) = 'directory'.

   make(Pathname) :-
      kernel:dir_make(Pathname).

   delete(Pathname) :-
      kernel:dir_delete(Pathname).

   delete_tree(Pathname) :-
      data.list:iter(files(Pathname)) {
         do('.') :-
            !.
         do('..') :-
            !.
         do(Filename) :-
            FullFilename = data.path:concat(Pathname, Filename),
            (
               check(FullFilename)
            ->
               delete_tree(FullFilename)
            ;
               io.file:delete(FullFilename)
            ).
      },
      delete(Pathname).

   change(Pathname) :-
      kernel:dir_change(Pathname).

   working(Pathname) :-
      kernel:dir_working(Pathname).

   current = module {
      get = working.

      set = change.
   }.

   files(Pathname, Files) :-
      kernel:dir_files(Pathname, Files).

   temp(Temp) :-
      (
         sys.env:{'TMPDIR'} = Temp
      ->
         true
      ;
         sys.env:{'TEMP'} = Temp
      ->
         true
      ;
         Temp = '/tmp'
      ).

   import: data.random.

   generate_temp(TemplateStr) = data.list:map(TemplateStr) {
      do('X') = ! >> data.list.(
         random_item(data.random, data.char.(('0' .. '9') ++ ('A' .. 'Z') ++ ('a' .. 'z')))
      ).

      do(C) = C.
   }.

   mktemp(Template, Dir) :-
      (
         sys.os:type = 'windows'
      ->
         data.atom:string(Template, TemplateStr),
         (
            repeat,
            generate_temp(TemplateStr, DirStr),
            Dir = data.string:atom(DirStr),
            catch(
               make(Dir),
               E,
               (
                  E = 'error'('system_error'(_), _),
                  sys.file:exists(Dir)
               ->
                  fail
               ;
                  throw(E)
               )
            )
         ->
            true
         )
      ;
         Dir = sys.process:call(data.atom:format('mktemp -d ~a', [Template]))
      ).

   mktemp = mktemp(data.path:concat(temp, 'tmp.XXXXXXXXXX'))
}
