module sys.os [Type] {
   /** architecture(Arch).
    *  Succeeds if <code>Arch</code> is the atom of the current architecture.
    *  (e.g. 'i686')
    */

   architecture = kernel:os_architecture.

   /** version(Ver).
    *  Succeeds if <code>Ver</code> is the atom of the OS version.
    *  (e.g. 'Linux 2.6.32-34-generic' or
    *  'Home Edition Service Pack 1 (Build 7601)').
    */

   version = kernel:os_version.

   /** type(Type).
    *  Succeeds if <code>Type</code> is the atom of the type of OS.
    *  (current defined values are 'linux' or 'windows').
    */

   type(Type') :-
      (
         data.var:check(Type)
      ->
         Type = version match (
            data.atom:concat('Linux ', _) -> 'linux'
         ;
            data.atom:concat('Darwin ', _) -> 'macos'
         ;
            data.atom:concat('Home ', _) -> 'windows'
         ;
            Unknown
         ->
            io.err:format(
               'Warning! Unknown OS \'~a\'.\n\
                Please define it in sys/os.mpl. Assuming linux.',
               [Unknown]
            ) >>
            'linux'
         )
      ;
         true
      ),
      Type' = Type.

   /** delimiter(Delim).
    *  Succeeds if <code>Delim</code> is the atom of the path delimiter.
    *  Typically, / on Unixes and \ on Windows.
    *  The use of data.path:delimiter/1 is more idiomatic.
    */

   path_delimiter = type match (
      'windows' -> '\\'
   ;
      _ -> '/'
   ).

   /** path_separator(Sep).
    *  Succeeds if <code>Sep</code> is the atom of the path separator in PATH
    *  environment variables.
    *  Typically, : on Unixes and ; on Windows.
    *  The use of data.path:separator/1 is more idiomatic.
    */

   path_separator = type match (
      'windows' -> ';'
   ;
      _ -> ':'
   ).
}
