module 'sys.process' [AtExit] {
   exec_filename(Filename) :-
      (
         Filename = sys.env:({'_'})
      ->
         true
      ;
         Filename = sys.cmdline:{0}
      ).

   at_exit(M) :-
      (
         data.var:check(AtExit)
      ->
         AtExit = data.ref.backtrackable:new([])
      ;
         true
      ),
      AtExit:set([M | AtExit:get]).

   import: module {
      flush_exit :-
         catch(
            (
               data.var:check(AtExit)
            ->
               true
            ;
               data.list:iter(AtExit:get) {
                  do(M) :-
                     M:do.
               }
            ->
               true
            ;
               io.err:print_endline('[Error] Failure on exit'),
               kernel:sys_exit(1)
            ),
            E,
            (
               io.err:format('[Error] Uncaught exception on exit: ~p\n', [E]),
               kernel:sys_exit(1)
            )
         ).
      }.

   exit(Error_code) :-
      flush_exit,
      kernel:sys_exit(Error_code).

   halt :-
      flush_exit,
      kernel:sys_halt.

   abort :-
      flush_exit,
      kernel:sys_abort.

   spawn(Command, Arguments, Error_code) :-
      kernel:sys_spawn(Command, Arguments, Error_code).

   spawn(Command, Arguments) :-
      sys.process:spawn(Command, Arguments, 0).

   exec(Command, In, Out, Err) :-
      kernel:sys_exec(Command, InFd, OutFd, ErrFd),
      io.stream.(
         In = of_kernel_out(InFd),
         Out = of_kernel_in(OutFd),
         Err = of_kernel_in(ErrFd)
      ).

   exec(Command, In, Out, Err, Pid) :-
      kernel:sys_exec(Command, InFd, OutFd, ErrFd, Pid),
      io.stream.(
         In = of_kernel_out(InFd),
         Out = of_kernel_in(OutFd),
         Err = of_kernel_in(ErrFd)
      ).

   wait(Pid, Status) :-
      kernel:sys_wait(Pid, Status).

   call(Command, In, Out) :-
      exec(Command, InStream, OutStream, ErrStream, Pid),
      InStream:print(In),
      Out = data.atom:remove_suffix(OutStream:atom, '\n'),
      (
         sys.os:type = 'windows'
      ->
         Status = 0 % wait is unsupported on windows
      ;
         wait(Pid, Status)
      ),
      InStream:close,
      OutStream:close,
      (
         Status = 0
      ->
         ErrStream:close
      ;
         ErrMsg = data.atom:remove_suffix(ErrStream:atom, '\n'),
         ErrStream:close,
         throw(
            'error'(
               'exec'(Command, ErrMsg, Status),
               '::'(sys.process, 'call/3')
            )
         )
      ).

   call(Command, Out) :-
      call(Command, '', Out).
}
